note
	description: "[
		File name abstraction
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_FILE_NAME

inherit
	FILE_NAME

create
	make,
	make_from_string,
	make_from_file

convert
	make_from_file ({FILE})

feature {NONE} -- Implementation

	make_from_file (a_file: FILE)
			-- Initialize with name of `a_file'.
		do
			make_from_string (a_file.name)
		ensure
			name_set: attached a_file.name as fn and then fn.is_equal (Current)
		end

feature -- Access

	directory_name: STRING
			-- Directory name.
			-- Example: For a file name "/foo/bar/spam.egg", the result is "/foo/bar".
		local
			last_delimiter_index: INTEGER
		do
			create Result.make_from_string (Current)

			from
				path_delimiters.start
			until
				path_delimiters.after
			loop
				last_delimiter_index := last_delimiter_index.max (Result.last_index_of (path_delimiters.item, Result.count))
				path_delimiters.forth
			end

			if last_delimiter_index > 0 then
				Result.remove_tail (Result.count - last_delimiter_index + 1)
			else
				Result := ""
			end
		end

	base_name: STRING
			-- Base name, including a potential extension.
			-- Example: For a file name "/foo/bar/spam.egg", the result is "spam.egg".
		do
			create Result.make_from_string (Current)

			from
				path_delimiters.start
			until
				path_delimiters.after
			loop
				Result.remove_head (Result.last_index_of (path_delimiters.item, Result.count))
				path_delimiters.forth
			end
		ensure
			non_empty_file_name: not Result.is_empty
			correct_file_name: ends_with (Result) and (
				Result.count = count or else path_delimiters.has (item (count - Result.count)))
		end

	base_name_without_extension: STRING
			-- Base name without extension, if there's any.
			-- Example: For a file name "/foo/bar/spam.egg", the result is "spam".
		do
			Result := base_name

			if has_extension then
				Result.remove_tail (extension.count + 1)
			end
		ensure then
			correct_name: base_name.starts_with (Result) and
				(has_extension implies base_name.count = Result.count + extension.count + 1) and
				(not has_extension implies base_name.count = Result.count)
		end

	extension: STRING
			-- File extension.
			-- Example: For a file name "/foo/bar/spam.egg", the result is "egg".
		require
			has_file_extension: has_extension
		do
			create Result.make_from_string (Current)

			Result.remove_head (Result.last_index_of ('.', Result.count))
		ensure
			non_empty_extension: not Result.is_empty
			correct_extension: item (count - Result.count).is_equal ('.') and ends_with (Result)
		end

feature -- Status report

	has_extension: BOOLEAN
			-- Is there a file extension?
		do
			Result := base_name.last_index_of ('.', base_name.count) > 1
		ensure
			result_correct: Result = (base_name.last_index_of ('.', base_name.count) > 1)
		end

feature -- Miscellaneous

	path_delimiters: LIST [CHARACTER]
			-- List of path delimiters used by the current `PLATFORM'.
		once
			create {LINKED_LIST [CHARACTER]} Result.make

			Result.compare_objects
			Result.extend ('/')

			if {PLATFORM}.is_windows then
				Result.extend ('%H')
			end
		end

end
