note
	description: "[
		Nodes in a trees that can be represented as HTML.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	EDL_HTML_NODE

feature -- Access

	out_indented (a_indentation_level: INTEGER): STRING
			-- HTML representation with an indentation level of `a_indentation_level'.
		require
			non_negative_indentation: a_indentation_level >= 0
		deferred
		end

	out_inline: STRING
			-- HTML representation without indentation and without additional newlines.
		deferred
		end

feature -- Miscellaneous

	escaped_text (a_string: STRING): STRING
			-- `a_string' with <, >, ', " and & escaped by appropriate HTML character entities.
		do
			create Result.make_from_string (a_string)

			Result.replace_substring_all ("&", "&amp;")
			Result.replace_substring_all ("<", "&lt;")
			Result.replace_substring_all (">", "&gt;")
			Result.replace_substring_all ("'", "&#039;")
			Result.replace_substring_all ("%"", "&quot;")
		end

	whitespace: CHARACTER = '%T'
			-- Character used to increase the indentation level of a string by 1.

	whitespaces (a_indentation_level: INTEGER): STRING
			-- Concatenation of `a_indentation_level' instances of `whitespace'.
		require
			non_negative_indentation: a_indentation_level >= 0
		do
			create Result.make_filled (whitespace, a_indentation_level)
		ensure
			correct_whitespaces: Result.is_equal (create {STRING}.make_filled (whitespace, a_indentation_level))
		end

end
