note
	description: "[
		Elements that depend on external source files.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	EDL_EXTERNAL_ELEMENT

inherit
	EDL_IDENTIFIABLE_ELEMENT
		redefine
			use_same_identifier,
			name
		end

feature {NONE} -- Initialization

	make (a_file: FILE)
			-- Initialize with a given file.
		require
			file_exists: a_file.exists
			file_is_readable: a_file.is_readable
			supported_file: supports_file (a_file)
		do
			file := a_file
		ensure
			file_set: file = a_file
		end

feature -- Access

	file: FILE
			-- File that contains the representation of this element.

	file_name: EDL_FILE_NAME
			-- Name of `file'.
		do
			Result := file
		ensure
			correct_result: attached file.name as fn and then fn.is_equal (Result)
		end

	name: STRING
			-- Base name of `file_name' without extension.
		do
			Result := file_name.base_name_without_extension
		ensure then
			correct_name: Result.is_equal (file_name.base_name_without_extension)
		end

feature -- Comparison

	use_same_identifier (a_other: like Current): BOOLEAN
			-- Is it ok to use the same identifier for both `Current'
			-- and `a_other'?
		do
			Result := file_name.is_equal (a_other.file_name)
		ensure then
			correct_result: file_name.is_equal (a_other.file_name) implies Result
		end

feature -- Miscellaneous

	supports_file (a_file: FILE): BOOLEAN
			-- Is `a_file' a supported file?
		do
			Result := attached (create {EDL_FILE_NAME}.make_from_file (a_file)) as fn and then (
				(fn.has_extension implies supports_file_extension (fn.extension)) and
				(not fn.has_extension implies supports_file_extension ("")))
		ensure
			correct_result: Result implies attached (create {EDL_FILE_NAME}.make_from_file (a_file)) as fn and then (
				(fn.has_extension implies supports_file_extension (fn.extension)) and
				(not fn.has_extension implies supports_file_extension ("")))
		end

	supports_file_extension (a_file_extension: STRING): BOOLEAN
			-- Is `a_file_extension' a supported file extension?
			-- The empty file extension represents the absence of a file extension.
		deferred
		end

invariant

	file_exists: file.exists
	file_is_readable: file.is_readable
	supported_file: supports_file (file)

end
