note
	description: "[
		Lists whose items are labeled using consecutive Roman numbers,
		starting with 'i'
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_ROMAN_LIST

inherit
	EDL_LIST
		redefine
			default_create,
			make_from_item,
			make_from_items,
			make_simple
		end

create
	default_create,
	make_from_item,
	make_from_items,
	make_simple

feature {NONE} -- Initialization

	default_create
			-- Initialize with no items.
		do
			Precursor
		ensure then
			not_upper_case: not is_upper_case
		end

	make_from_item (a_item: EDL_LIST_ITEM)
			-- Initialize with one item `a_item'.
		do
			Precursor (a_item)
		ensure then
			not_upper_case: not is_upper_case
		end

	make_from_items (a_items: ARRAY [EDL_LIST_ITEM])
			-- Initialize with items from `a_items'.
		do
			Precursor (a_items)
		ensure then
			not_upper_case: not is_upper_case
		end

	make_simple (a_strings: INDEXABLE [STRING, INTEGER])
			-- Initialize with `a_strings'.
		do
			Precursor (a_strings)
		ensure then
			not_upper_case: not is_upper_case
		end

feature -- Access

	is_upper_case: BOOLEAN assign set_upper_case
			-- Are items labeled using upper-case roman numbers?

feature -- Element change

	set_upper_case (a_boolean: BOOLEAN)
			-- Set `is_upper_case' to `a_boolean'.
		do
			is_upper_case := a_boolean
		ensure
			is_upper_case_set: is_upper_case = a_boolean
		end

feature -- Miscellaneous

	accept (a_visitor: EDL_VISITOR)
			-- Accept `a_visitor' and pass it on to the children.
		do
			a_visitor.visit_roman_list (Current)
			accept_children (a_visitor)
			a_visitor.leave_roman_list (Current)
		end

end
