note
	description: "[
		Table cells are used in table rows and contain autonomous elements.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_TABLE_CELL

inherit
	EDL_COMPOUND_ELEMENT [EDL_AUTONOMOUS_ELEMENT]

create
	default_create,
	make_from_element,
	make_from_elements,
	make_simple,
	make_simple_heading

feature {NONE} -- Initialization

	make_simple (a_string: STRING)
			-- Initialize with a paragraph containing only a text element with
			-- `a_string' as content.
		require
			string_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_string)
		do
			default_create
			extend_simple (a_string)
		ensure
			not_heading: not is_heading
			one_child: count = 1
		end

	make_simple_heading (a_string: STRING)
			-- Initialize with a paragraph containing only a text element with
			-- `a_string' as content. Set the cell as heading.
		require
			string_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_string)
		do
			make_simple (a_string)
			is_heading := True
		ensure
			heading: is_heading
			one_child: count = 1
		end

feature -- Access

	is_heading: BOOLEAN assign set_heading
		-- Is this a heading cell?

feature -- Element change

	extend_simple (a_string: STRING)
			-- Add a paragraph containing only a text element with
			-- `a_string' as content.
		require
			string_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_string)
		do
			extend (create {EDL_PARAGRAPH}.make_simple (a_string))
		ensure
			correct_count: count = old count + 1
		end

	set_heading (a_boolean: BOOLEAN)
			-- Change the heading status to `a_boolean'.
		do
			is_heading := a_boolean
		ensure
			is_heading_set: is_heading = a_boolean
		end

feature -- Miscellaneous

	accept (a_visitor: EDL_VISITOR)
			-- Accept `a_visitor' and pass it on to the children.
		do
			a_visitor.visit_table_cell (Current)
			accept_children (a_visitor)
			a_visitor.leave_table_cell (Current)
		end

end
