note
	description: "[
		Table of contents contain a list of all section titles and
		the corresponding section numbers. Optionally, the list
		only contains section with a section nesting depth up to
		a specified limit.
		]"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_TABLE_OF_CONTENTS

inherit
	EDL_AUTONOMOUS_ELEMENT
		redefine
			default_create
		end

create
	default_create,
	make_limited_depth

feature {NONE} -- Initialization

	default_create
			-- Initialize without depth limit.
		do
			depth := (create {INTEGER}).max_value
			title := "Table of contents"
		ensure then
			default_title: ("Table of contents").is_equal (title)
		end

	make_limited_depth (a_depth: INTEGER)
			-- Initialize with a maxium depth.
		require
			positive_depth: a_depth > 0
		do
			depth := a_depth
			title := "Table of contents"
		ensure
			default_title: ("Table of contents").is_equal (title)
			depth_set: depth = a_depth
		end

feature -- Access

	title: detachable STRING assign set_title
			-- Title of the table of contents, if availabe.

	depth: INTEGER assign set_depth
			-- Depth of the table of contents. Only section with a
			-- section nesting depth of `depth' or less will have
			-- an entry in this table of contents.

feature -- Element change

	set_title (a_title: detachable STRING)
			-- Change `title' to `a_title'. If `a_title' is Void,
			-- the title is removed.
		require
			title_is_utf8: attached a_title as attached_title implies
				(create {EDL_UTF8_ENCODING}).is_valid (attached_title)
		do
			title := a_title
		ensure
			title_set: title = a_title
		end

	set_depth (a_depth: INTEGER)
			-- Change `depth' to `a_depth'.
		require
			positive_depth: a_depth > 0
		do
			depth := a_depth
		ensure
			depth_set: depth = a_depth
		end

feature -- Miscellaneous

	accept (a_visitor: EDL_VISITOR)
			-- Accept `a_visitor'.
		do
			a_visitor.visit_table_of_contents (Current)
			a_visitor.leave_table_of_contents (Current)
		end

invariant

	positive_depth: depth > 0
	title_is_utf8: attached title as attached_title implies
		(create {EDL_UTF8_ENCODING}).is_valid (attached_title)

end
