note
	description: "[
		Table rows are used in tables and contain table cells.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_TABLE_ROW

inherit
	EDL_COMPOUND_ELEMENT [EDL_TABLE_CELL]
		rename
			make_from_elements as make_from_cells,
			make_from_element as make_from_cell,
			extend_with_elements as extend_with_cells
		redefine
			is_valid_child
		end

create
	default_create,
	make_from_cell,
	make_from_cells,
	make_simple,
	make_simple_header

feature {NONE} -- Initialization

	make_simple (a_strings: INDEXABLE [STRING, INTEGER])
			-- Initialize with `a_strings', placing each string
			-- in a table cell.
		require
			strings_are_utf8: are_strings_utf8 (a_strings)
		do
			default_create
			extend_with_strings (a_strings, agent (a_string: STRING): EDL_TABLE_CELL
				do
					create Result.make_simple (a_string)
				end)
		ensure
			correct_count: attached a_strings.index_set as index_set implies count = index_set.count
		end

	make_simple_header (a_strings: INDEXABLE [STRING, INTEGER])
			-- Initialize with `a_strings', placing each string
			-- in a heading table cell.
		require
			strings_are_utf8: are_strings_utf8 (a_strings)
		do
			default_create
			extend_with_strings (a_strings, agent (a_string: STRING): EDL_TABLE_CELL
				do
					create Result.make_simple_heading (a_string)
				end)
		ensure
			correct_count: attached a_strings.index_set as index_set implies count = index_set.count
		end

feature -- Element change

	extend_simple (a_string: STRING)
			-- Add a cell containing a paragraph with just a text element with
			-- `a_string' as content.
		require
			string_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_string)
			at_most_one_row: attached {EDL_TABLE} parent as table implies table.count = 1
		do
			extend (create {EDL_TABLE_CELL}.make_simple (a_string))
		ensure
			correct_count: count = old count + 1
		end

feature -- Miscellaneous

	is_valid_child (a_cell: EDL_TABLE_CELL): BOOLEAN
			-- Is it ok to add `a_cell'? Answer: Only if this row is not yet
			-- added to a table or if it is the only row in that table.
		do
			Result := attached {EDL_TABLE} parent as table implies table.count = 1
		ensure then
			valid_if_at_most_one_row: Result = (attached {EDL_TABLE} parent as table implies table.count = 1)
		end

	accept (a_visitor: EDL_VISITOR)
			-- Accept `a_visitor' and pass it on to the children.
		do
			a_visitor.visit_table_row (Current)
			accept_children (a_visitor)
			a_visitor.leave_table_row (Current)
		end

end
