note
	description: "[
		Visitors used to determine the section number for each section in a document.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_SECTION_NUMBER_COLLECTOR

inherit
	EDL_ELEMENT_COLLECTOR [EDL_SECTION]
		redefine
			visit_document,
			visit_section,
			leave_section,
			default_create
		end

create
	default_create

feature {NONE} -- Inititialization

	default_create
			-- Create empty `last_numbers' and ensure that "1"
			-- is used as the section number for the first visited section.
		do
			Precursor

			create last_numbers.make (0)
			create current_counters.make

			current_counters.extend (0)
		ensure then
			empty_numbers: last_numbers.is_empty
			current_counter_level: current_counters.count = 1
			current_counter_value: current_counters.last = 0
		end

feature -- Access

	last_numbers: HASH_TABLE [STRING, EDL_SECTION]

feature {EDL_ELEMENT} -- Visitor

	visit_document (a_document: EDL_DOCUMENT)
			-- Create empty `last_numbers' and ensure that "1"
			-- is used as the section number for the first visited section.
		do
			default_create
		ensure then
			empty_numbers: last_numbers.is_empty
			current_counter_level: current_counters.count = 1
			current_counter_value: current_counters.last = 0
		end

	visit_section (a_section: EDL_SECTION)
			-- Increment the i-th entry of `current_counters' by one
			-- given `a_section' with nesting depth i.
			-- Add the section number of `a_section' to `last_numbers'.
		local
			count: INTEGER
		do
			Precursor (a_section)

			count := current_counters.count
			current_counters [count] := current_counters [count] + 1
			last_numbers [a_section] := current_number

			current_counters.extend (0)
		ensure then
			incremented_counter_level: current_counters.count = old current_counters.count + 1
			current_counter_value: current_counters.last = 0
			last_numbers_updated: last_numbers.has (a_section)
		end

	leave_section (a_section: EDL_SECTION)
			-- Remove the last entry of `current_counters'.
		do
			Precursor (a_section)

			current_counters.finish
			current_counters.remove
		ensure then
			decremented_counter_level: current_counters.count = old current_counters.count - 1
		end

feature {NONE} -- Implementation

	current_counters: TWO_WAY_LIST [INTEGER]
			-- The i-th entry of this list corresponds to the current value
			-- of the section counter of sections with nesting depth i.

	current_number: STRING
			-- The string representation of the current section number,
			-- where the individual section counters are separated by '.'.
		do
			create Result.make_empty

			from
				current_counters.start
			until
				current_counters.after
			loop
				if current_counters.index /= 1 then
					Result.append_character ('.')
				end

				Result.append_integer (current_counters.item)

				current_counters.forth
			end
		ensure
			correct_dot_count: Result.occurrences ('.') = current_counters.count - 1
		end

end
