note
	description: "[
		Visitors that redirect all calls to visit_* to visit_element
		and calls to leave_* to leave_element respectively, 
		unless redefined by subclasses.
		Useful for visitors that need to apply a certain operation to
		all elements.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	EDL_SINK_VISITOR

inherit
	EDL_VISITOR

feature {EDL_VISITABLE} -- Visitor

	visit_element (a_element: EDL_ELEMENT)
			-- Visit `a_element'.
		deferred
		end

	leave_element (a_element: EDL_ELEMENT)
			-- Leave `a_element'.
		deferred
		end

	visit_text_element (a_text_element: EDL_TEXT_ELEMENT)
			-- Visit `a_text_element'.
		do
			visit_element (a_text_element)
		end

	leave_text_element (a_text_element: EDL_TEXT_ELEMENT)
			-- Leave `a_text_element'.
		do
			leave_element (a_text_element)
		end

	visit_table (a_table: EDL_TABLE)
			-- Visit `a_table'.
		do
			visit_element (a_table)
		end

	leave_table (a_table: EDL_TABLE)
			-- Leave `a_table'.
		do
			leave_element (a_table)
		end

	visit_table_row (a_table_row: EDL_TABLE_ROW)
			-- Visit `a_table_row'.
		do
			visit_element (a_table_row)
		end

	leave_table_row (a_table_row: EDL_TABLE_ROW)
			-- Leave `a_table_row'.
		do
			leave_element (a_table_row)
		end

	visit_table_cell (a_table_cell: EDL_TABLE_CELL)
			-- Visit `a_table_cell'.
		do
			visit_element (a_table_cell)
		end

	leave_table_cell (a_table_cell: EDL_TABLE_CELL)
			-- Leave `a_table_cell'.
		do
			leave_element (a_table_cell)
		end

	visit_alphabetic_list (a_alphabetic_list: EDL_ALPHABETIC_LIST)
			-- Visit `a_alphabetic_list'.
		do
			visit_element (a_alphabetic_list)
		end

	leave_alphabetic_list (a_alphabetic_list: EDL_ALPHABETIC_LIST)
			-- Leave `a_alphabetic_list'.
		do
			leave_element (a_alphabetic_list)
		end

	visit_arabic_list (a_arabic_list: EDL_ARABIC_LIST)
			-- Visit `a_arabic_list'.
		do
			visit_element (a_arabic_list)
		end

	leave_arabic_list (a_arabic_list: EDL_ARABIC_LIST)
			-- Leave `a_arabic_list'.
		do
			leave_element (a_arabic_list)
		end

	visit_bullet_list (a_bullet_list: EDL_BULLET_LIST)
			-- Visit `a_bullet_list'.
		do
			visit_element (a_bullet_list)
		end

	leave_bullet_list (a_bullet_list: EDL_BULLET_LIST)
			-- Leave `a_bullet_list'.
		do
			leave_element (a_bullet_list)
		end

	visit_roman_list (a_roman_list: EDL_ROMAN_LIST)
			-- Visit `a_roman_list'.
		do
			visit_element (a_roman_list)
		end

	leave_roman_list (a_roman_list: EDL_ROMAN_LIST)
			-- Leave `a_roman_list'.
		do
			leave_element (a_roman_list)
		end

	visit_list_item (a_list_item: EDL_LIST_ITEM)
			-- Visit `a_list_item'.
		do
			visit_element (a_list_item)
		end

	leave_list_item (a_list_item: EDL_LIST_ITEM)
			-- Leave `a_list_item'.
		do
			leave_element (a_list_item)
		end

	visit_section (a_section: EDL_SECTION)
			-- Visit `a_section'.
		do
			visit_element (a_section)
		end

	leave_section (a_section: EDL_SECTION)
			-- Leave `a_section'.
		do
			leave_element (a_section)
		end

	visit_image (a_image: EDL_IMAGE)
			-- Visit `a_image'.
		do
			visit_element (a_image)
		end

	leave_image (a_image: EDL_IMAGE)
			-- Leave `a_image'.
		do
			leave_element (a_image)
		end

	visit_paragraph (a_paragraph: EDL_PARAGRAPH)
			-- Visit `a_paragraph'.
		do
			visit_element (a_paragraph)
		end

	leave_paragraph (a_paragraph: EDL_PARAGRAPH)
			-- Leave `a_paragraph'.
		do
			leave_element (a_paragraph)
		end

	visit_reference (a_reference: EDL_REFERENCE)
			-- Visit `a_reference'.
		do
			visit_element (a_reference)
		end

	leave_reference (a_reference: EDL_REFERENCE)
			-- Leave `a_reference'.
		do
			leave_element (a_reference)
		end

	visit_table_of_contents (a_table_of_contents: EDL_TABLE_OF_CONTENTS)
			-- Visit `a_table_of_contents'.
		do
			visit_element (a_table_of_contents)
		end

	leave_table_of_contents (a_table_of_contents: EDL_TABLE_OF_CONTENTS)
			-- Visit `a_table_of_contents'.
		do
			leave_element (a_table_of_contents)
		end

	visit_raw_element (a_raw_element: EDL_RAW_ELEMENT)
			-- Visit a `a_raw_element'.
		do
			visit_element (a_raw_element)
		end

	leave_raw_element (a_raw_element: EDL_RAW_ELEMENT)
			-- Leave a `a_raw_element'.
		do
			leave_element (a_raw_element)
		end

	visit_listing (a_listing: EDL_LISTING)
			-- Visit a `a_listing'.
		do
			visit_element (a_listing)
		end

	leave_listing (a_listing: EDL_LISTING)
			-- Leave a `a_listing'.
		do
			leave_element (a_listing)
		end

	visit_document (a_document: EDL_DOCUMENT)
			-- Visit `a_document'.
		do
			visit_element (a_document)
		end

	leave_document (a_document: EDL_DOCUMENT)
			-- Leave `a_document'.
		do
			leave_element (a_document)
		end

end
