note
	description: "[
		Lists represent one-dimensionally structured data.
		Each item of a list may contain autonomous elements,
		including other lists.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	EDL_LIST

inherit
	EDL_COMPOUND_ELEMENT [EDL_LIST_ITEM]
		rename
			make_from_elements as make_from_items,
			make_from_element as make_from_item,
			extend_with_elements as extend_with_item
		end

	EDL_AUTONOMOUS_ELEMENT
		undefine
			default_create,
			has
		end

feature {NONE} -- Initialization

	make_simple (a_strings: INDEXABLE [STRING, INTEGER])
			-- Initialize with a single item for each string in `a_strings'.
		require
			strings_are_utf8: are_strings_utf8 (a_strings)
		do
			default_create
			extend_with_strings (a_strings, agent (a_string: STRING): EDL_LIST_ITEM
				do
					create Result.make_simple (a_string)
				end)
		ensure
			correct_count: attached a_strings.index_set as index_set implies count = index_set.count
		end

feature -- Element change

	extend_simple (a_string: STRING)
			-- Add a list item containing a paragraph with just a text element with
			-- `a_string' as content.
		require
			string_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_string)
		do
			extend (create {EDL_LIST_ITEM}.make_simple (a_string))
		ensure
			correct_count: count = old count + 1
		end

end
