note
	description: "[
		Elements with a title to structure documents.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_SECTION

inherit
	EDL_COMPOUND_ELEMENT [EDL_AUTONOMOUS_ELEMENT]
		rename
			make_from_elements as ce_make_from_elements,
			make_from_element as ce_make_from_element
		end

	EDL_IDENTIFIABLE_ELEMENT
		rename
			name as title
		undefine
			default_create,
			has
		end

	EDL_AUTONOMOUS_ELEMENT
		undefine
			default_create,
			has
		end

create
	make,
	make_from_element,
	make_from_elements

feature -- Initialization

	make (a_title: STRING)
			-- Initialize a new section with `a_title'.
		require
			title_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_title)
		do
			default_create
			title := a_title
		ensure
			title_set: title = a_title
			no_children: count = 0
		end

	make_from_element (a_title: STRING; a_element: EDL_AUTONOMOUS_ELEMENT)
			-- Initialize with `a_title' and a single child `a_element'.
		require
			title_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_title)
			element_has_no_parent: a_element.parent = Void
			valid_child: is_valid_child (a_element)
		do
			make (a_title)
			ce_make_from_element (a_element)
		ensure
			correct_count: count = 1
			title_set: title = a_title
			parent_set: a_element.parent = Current
		end

	make_from_elements (a_title: STRING; a_elements: ARRAY [EDL_AUTONOMOUS_ELEMENT])
			-- Initialize with `a_title' and children elements in `a_elements'.
		require
			title_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_title)
			elements_have_no_parent: a_elements.for_all (agent (a_element: EDL_AUTONOMOUS_ELEMENT): BOOLEAN
				do
					Result := a_element.parent = Void
				end)
			valid_children: are_valid_children (a_elements)
		do
			make (a_title)
			ce_make_from_elements (a_elements)
		ensure
			correct_count: count = a_elements.count
			title_set: title = a_title
			parent_set: a_elements.for_all (agent (a_element: EDL_AUTONOMOUS_ELEMENT): BOOLEAN
				do
					Result := a_element.parent = Current
				end)
		end

feature -- Access

	title: STRING assign set_title
			-- Title of the section.

feature -- Element change

	set_title (a_title: STRING)
			-- Change `title' to `a_title'.
		require
			title_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_title)
		do
			title := a_title
		ensure
			title_set: title = a_title
		end

feature -- Miscellaneous

	accept (a_visitor: EDL_VISITOR)
			-- Accept `a_visitor' and pass it on to the children.
		do
			a_visitor.visit_section (Current)
			accept_children (a_visitor)
			a_visitor.leave_section (Current)
		end

invariant

	title_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (title)

end
