note
	description: "[
		Visitors used to find all elements of a certain type in a document.
		All found elements are both provided as a forest and a flat list.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_ELEMENT_COLLECTOR [G -> EDL_ELEMENT]

inherit
	EDL_SINK_VISITOR
		redefine
			default_create,
			visit_document
		end

create
	default_create

feature {NONE} -- Initialization

	default_create
			-- Create an empty `last_forest' and an empty `last_list'.
		do
			create last_forest.make
			create last_list.make
			current_tree := Void
		ensure then
			empty_forest: last_forest.is_empty
			empty_list: last_list.is_empty
		end

feature -- Basic operations

	collect (a_document: EDL_DOCUMENT)
			-- Collect all elements of type `G' in `a_document'.
		do
			a_document.accept (Current)
		end

feature -- Access

	last_forest: LINKED_LIST [LINKED_TREE [G]]
			-- List of trees containing all elements of type `G' in the
			-- previously visited document.

	last_list: LINKED_LIST [G]
			-- List of all elements of type `G' in the previously
			-- visited document.

feature {EDL_ELEMENT} -- Visitor

	visit_element (a_element: EDL_ELEMENT)
			-- Put `a_element' into `last_forest' if it's of a type
			-- that conforms to `G'.
		local
			new_tree: LINKED_TREE [G]
		do
			if attached {G} a_element as a_g_element then
				last_list.extend (a_g_element)

				create new_tree.make (a_g_element)

				if attached current_tree as attached_current_tree then
					attached_current_tree.put_child (new_tree)
				else
					last_forest.extend (new_tree)
				end

				current_tree := new_tree
			end
		ensure then
			increased_count: (attached {G} a_element as a_g_element) implies last_list.count = old last_list.count + 1
			right_last_item: (attached {G} a_element as a_g_element) implies last_list.last = a_g_element
		end

	leave_element (a_element: EDL_ELEMENT)
			-- Update `current_tree' if `a_element' is of a type that conforms to `G'.
		do
			if attached {G} a_element as a_g_element then
				if attached current_tree as attached_current_tree then
					current_tree := attached_current_tree.parent
				end
			end
		ensure then
			same_list: last_list.is_equal (old last_list)
			same_forest: last_forest.is_equal (old last_forest)
		end

	visit_document (a_document: EDL_DOCUMENT)
			-- Create an empty `last_forest' and an empty `last_list'.
		do
			default_create
		ensure then
			empty_forest: last_forest.is_empty
			empty_list: last_list.is_empty
		end

feature {NONE} -- Implementation

	current_tree: detachable LINKED_TREE [G]

end
