note
	description: "Summary description for {EDL_HTML_TEXT_NODE}."
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_HTML_TEXT_NODE

inherit
	EDL_HTML_NODE

create
	make,
	make_unescaped

feature {NONE} -- Initialization

	make (a_text: STRING)
			-- Initialize with `a_text' as content.
		do
			make_unescaped (escaped_text (a_text))
		ensure
			not_undistorted: not is_undistorted
		end

	make_unescaped (a_unescaped_text: STRING)
			-- Initialize with `a_text' escaped as content.
		do
			content := a_unescaped_text
		ensure
			not_undistorted: not is_undistorted
		end

feature -- Access

	out_indented (a_indentation_level: INTEGER): STRING
			-- Indented string representation.
		do
			create Result.make_empty
			if not is_undistorted then
				Result.append (whitespaces (a_indentation_level))
			end
			Result.append (out_inline)
		end

	out_inline: STRING
			-- Inline string representation.
		do
			create Result.make_from_string (content)
			if not is_undistorted then
				Result.replace_substring_all ("%R%N", " ")
				Result.replace_substring_all ("%R", " ")
				Result.replace_substring_all ("%N", " ")
				Result.replace_substring_all ("%T", " ")
			end
		end

	is_undistorted: BOOLEAN assign set_undistorted
			-- Does the node need to be completely undistorted (e.g. no indentation)?

feature -- Element change

	set_undistorted (a_boolean: BOOLEAN)
			-- Set `is_undistorted' to `a_boolean'.
		do
			is_undistorted := a_boolean
		ensure
			undistorted_set: is_undistorted = a_boolean
		end

feature {NONE} -- Implementation

	content: STRING
			-- Undistorted text content.

end
