note
	description: "[
		Objects that provide combined information about all documents in a document
		collection and manage the names of files created in the output directory.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_HTML_TRANSLATOR_CONTEXT

inherit
	EDL_TEXT_TRANSLATOR_CONTEXT
		rename
			make as ttc_make
		redefine
			document_file_extension
		end

create
	make

feature {NONE} -- Initialization

	make (a_document_collection: EDL_DOCUMENT_COLLECTION; a_writer: EDL_HTML_WRITER)
			-- Initialize with `a_document_collection', collecting any information
			-- that is needed later on and set `writer' to `a_writer'.
		do
			ttc_make (a_document_collection)
			writer := a_writer
		ensure
			writer_set: writer = a_writer
		end

feature -- Access

	writer: EDL_HTML_WRITER
			-- Current latex writer object.

feature -- Basic operations

	resolve_reference (a_reference: EDL_REFERENCE): STRING
			-- Encoded URI to be used in the HTML output for `a_reference'.
		do
			create Result.make_empty

			if attached a_reference.destination_uri as uri then
				Result := uri
			elseif attached a_reference.destination_section as section then
				check attached section.parent_document as document then
					Result.append (document_file_name (document))
					Result.append ("#")
					Result.append (section_id (section))
				end
			elseif attached a_reference.destination_document as document then
				Result := document_file_name (document)
			end
		end

	css_file_name: STRING = "style.css"
			-- File name of the CSS file.

feature {NONE} -- Implementation

	document_file_extension: STRING = "html"
			-- File extension of HTML files.

end
