note
	description: "[
		Hierarchical structures of elements of various types, 
		holding user-defined data.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_DOCUMENT

inherit
	EDL_COMPOUND_ELEMENT [EDL_AUTONOMOUS_ELEMENT]
		rename
			make_from_elements as ce_make_from_elements,
			make_from_element as ce_make_from_element
		end

	EDL_IDENTIFIABLE_ELEMENT
		rename
			name as title
		undefine
			default_create,
			has
		end

create
	make,
	make_from_element,
	make_from_elements

feature {NONE} -- Initialization

	make (a_title: STRING)
			-- Initialize with `a_title'.
		require
			title_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_title)
		do
			default_create
			title := a_title
		ensure
			no_children: count = 0
			title_set: title = a_title
		end

	make_from_element (a_title: STRING; a_element: EDL_AUTONOMOUS_ELEMENT)
			-- Initialize with `a_title' and a single child `a_element'.
		require
			title_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_title)
			element_has_no_parent: a_element.parent = Void
			valid_child: is_valid_child (a_element)
		do
			make (a_title)
			ce_make_from_element (a_element)
		ensure
			correct_count: count = 1
			title_set: title = a_title
			parent_set: a_element.parent = Current
		end

	make_from_elements (a_title: STRING; a_elements: ARRAY [EDL_AUTONOMOUS_ELEMENT])
			-- Initialize with `a_title' and children elements in `a_elements'.
		require
			title_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_title)
			elements_have_no_parent: a_elements.for_all (agent (a_element: EDL_AUTONOMOUS_ELEMENT): BOOLEAN
				do
					Result := a_element.parent = Void
				end)
			valid_children: are_valid_children (a_elements)
		do
			make (a_title)
			ce_make_from_elements (a_elements)
		ensure
			correct_count: count = a_elements.count
			title_set: title = a_title
			parent_set: a_elements.for_all (agent (a_element: EDL_AUTONOMOUS_ELEMENT): BOOLEAN
				do
					Result := a_element.parent = Current
				end)
		end

feature -- Access

	title: STRING assign set_title
			-- Title of the document.

	author: detachable STRING assign set_author
			-- Name of the author of the document.

	date: detachable DT_DATE assign set_date
			-- Creation date of the document.

feature -- Element change

	set_title (a_title: STRING)
			-- Change `title' to `a_title'.
		require
			title_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_title)
		do
			title := a_title
		ensure
			title_set: title = a_title
		end

	set_author (a_author: STRING)
			-- Set `author' to `a_author'.
		require
			author_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (a_author)
		do
			author := a_author
		ensure
			author_set: author = a_author
		end

	set_date (a_date: DT_DATE)
			-- Set `date' to `a_date'.
		do
			date := a_date
		ensure
			date_set: date = a_date
		end

feature -- Status report

	fulfills_constraint (a_constraint: EDL_DOCUMENT_CONSTRAINT): BOOLEAN
			-- Is `a_constraint' fulfilled?
		do
			a_constraint.validate (Current)
			Result := a_constraint.last_result
		end

feature -- Miscellaneous

	accept (a_visitor: EDL_VISITOR)
			-- Accept `a_visitor' and pass it on to the children.
		do
			a_visitor.visit_document (Current)
			accept_children (a_visitor)
			a_visitor.leave_document (Current)
		end

invariant

	title_is_utf8: (create {EDL_UTF8_ENCODING}).is_valid (title)
	author_is_utf8: attached author as a implies (create {EDL_UTF8_ENCODING}).is_valid (a)

end
