note
	description: "[
		Base type of objects that can be used to create documents.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	EDL_ELEMENT

inherit
	EDL_VISITABLE
		undefine
			copy,
			is_equal
		end

	HASHABLE
		undefine
			copy,
			is_equal
		end

	IDENTIFIED
		rename
			object_id as hash_code
		end

feature -- Access

	parent: detachable EDL_COMPOUND_ELEMENT [EDL_ELEMENT] assign set_parent
			-- Parent if available.

	parent_document: detachable EDL_DOCUMENT
			-- Parent document if available.
		local
			found: BOOLEAN
			element: EDL_ELEMENT
		do
			from
				element := Current
			until
				found
			loop
				if attached {EDL_DOCUMENT} element as document then
					Result := document
					found := True
				else
					if attached element.parent as p then
						element := p
					else
						found := True
					end
				end
			end
		end

	has (a_element: EDL_ELEMENT): BOOLEAN
			-- Is `a_element' equivalent to `Current'
			-- or a heir element of `Current' (by reference)?
		do
			Result := Current = a_element
		end

feature {EDL_COMPOUND_ELEMENT} -- Element change

	set_parent (a_element: EDL_COMPOUND_ELEMENT [EDL_ELEMENT])
			-- Change the parent to `a_element'.
		require
			element_has_current: a_element.has (Current)
		do
			parent := a_element
		ensure
			parent_set: parent = a_element
		end

invariant

	parent_document_implies_parent: (attached parent_document implies attached parent) or attached {EDL_DOCUMENT} Current

end
