note
	description: "[
		Constraints on the nesting depth of certain elements in a document.
		]"
	author: "Bernhard Brodowsky, Severin Heiniger and Stefan Heule"
	date: "$Date$"
	revision: "$Revision$"

class
	EDL_NESTING_DEPTH_CONSTRAINT [G -> EDL_ELEMENT]

inherit
	EDL_DOCUMENT_CONSTRAINT
	EDL_SINK_VISITOR
		redefine
			visit_document,
			leave_document
		end

create
	make

feature {NONE} -- Initialization

	make (a_nesting_depth_limit: INTEGER)
			-- Initialize with `a_nesting_depth_limit'.
		require
			positive_nesting_depth_limit: a_nesting_depth_limit > 0
		do
			nesting_depth_limit := a_nesting_depth_limit
		ensure
			nesting_depth_limit_set: nesting_depth_limit = a_nesting_depth_limit
		end

feature {EDL_VISITABLE} -- Visitor

	visit_element (a_element: EDL_ELEMENT)
			-- Increment `current_nesting_depth' if `a_element' conforms to `G' and
			-- set the result to False if the nesting is too deep.
		do
			if attached {G} a_element then
				current_nesting_depth := current_nesting_depth + 1

				if current_nesting_depth > nesting_depth_limit then
					last_result := False
				end
			end
		ensure then
			incremented_current_nesting_depth:
				attached {G} a_element implies
				current_nesting_depth = old current_nesting_depth + 1
			valid_last_result: current_nesting_depth > nesting_depth_limit implies not last_result
			constraint_violation_not_lost: last_result implies old last_result
		end

	leave_element (a_element: EDL_ELEMENT)
			-- Decrement `current_nesting_depth' if `a_element' conforms to `G'.
		do
			if attached {G} a_element then
				current_nesting_depth := current_nesting_depth - 1
			end
		ensure then
			decremented_current_nesting_depth:
				attached {G} a_element implies current_nesting_depth = old current_nesting_depth - 1
			constraint_violation_not_lost: last_result implies old last_result
		end

	visit_document (a_document: EDL_DOCUMENT)
			-- Set `last_result' to True and `current_nesting_depth' to 0.
		do
			current_nesting_depth := 0
			last_result := True
		ensure then
			current_nesting_depth_reset: current_nesting_depth = 0
			document_fulfills_constraint_by_default: last_result
		end

	leave_document (a_document: EDL_DOCUMENT)
			-- Do nothing.
		do
		ensure then
			current_nesting_depth_reset: current_nesting_depth = 0
		end

feature {NONE} -- Implementation

	current_nesting_depth: INTEGER
			-- Nesting depth of elements of type `G' in the current traversal of the given document.

	nesting_depth_limit: INTEGER
			-- Maximum nesting depth as specified by the client.

invariant

	non_negative_current_nesting_depth: current_nesting_depth >= 0
	positive_nesting_depth_limit: nesting_depth_limit > 0

end
